class MSBController extends PlayerController
config (MSBCode);

var bool bHasInteraction;   //for key bindings and Mouse
var MSBInterAction MInteract;
var MSBMouseOverlay Mouse;  // the Mouse
var MSBMiniMap MiniMap;     //the Mini Map

var config array<string> ContextButtonList;    //for Context Menu= right button click
var config array<string> ContextButtonCommand; //the command to be executed from the context menu
var config array<string> ContextButtonName;    //what to write on it

var() globalconfig array<string> SaveGames;          //list of savegames as saved in MSBCode.ini
var() globalconfig array<string> SaveUTCharacters;   //list of UT characters
var() globalconfig array<string> SaveCharacters;     //list of saved Voxel characters as saved in MSBCode.ini
var() globalconfig array<string> SaveCharParts;      //list of saved Voxel parts as saved in MSBCode.ini
var() globalconfig array<string> Save2DCharacters;   //list of saved 2D charactersas saved in MSBCode.ini
var() globalconfig array<string> SaveProps;          //tbd
var() globalconfig array<string> SaveAnimations;     //tbd
var() globalconfig array<string> SavePoses;          //list of saved Poses as saved in MSBCode.ini

var() globalconfig array<string> VoxelShader;        //list of available Voxelshaders as saved in MSBCode.ini

var() globalconfig float MouseSens;                  //Mouse sensitivity

var() globalconfig bool bGrid;                       //enable Grid
var() config int Gridspacing;                  //Gridspacing


//Lists for quickly finding MSB specific objects
var array<MSBScriptNodeButton> LevelMSBScriptNodes;
var array<MSBNodeButton> LevelNodes;
var array<MSBExtendedListButton> LevelComplexButtons;
var array<MSBActor> LevelActors;
var array<MSBPawn> LevelPawns;
var array<MSBContainerButton> LevelContainers;
var array <Actor> LevelSpecial;

// selecting and showing references
var array <MSBPointActor> pointers;
var array <MSBSelectPointActor> SelectPointers;
var bool canSelect;

//basic thruths
var bool bPlay;                                //we are simulating/playing
var bool bMove;                                //we are flying through space
var bool bWarningMessageDisplayed;             //there's a warning Message displayed
//command line options
var bool bInstantRun, bInstantLoad;           //see MSBGame
var string instantLevel;                      //Level Name to instantly load (for self running demos, films, etc.

//Render to disk stuff
var bool bStartRender;
var string filename;

var float RunTime;                           //how long is the scene running already?

var int screenX,                             //Screen Size in pixels x=width, y=height
        screenY;

var int scrollLeft;                          //MiniMap functionality
var int scrollUp;

var() vector StartLoc; //moved from VoxelEdController
var() rotator StartRot;

// ********************************************************************************
// initialize rectangular Pointers (the red things when you select stuff)
// and count the time passed when running (used for recording tbd)
// ********************************************************************************
event PostBeginPlay()
{
local int i;

super.PostBeginPlay();

pointers[i]=Spawn(class'MSBPointActor',,,Location);
pointers[i].bHidden=true;
selectPointers[i]=Spawn(class'MSBSelectPointActor',,,Location);
selectPointers[i].bHidden=true;
}

function PlayerTick(float DeltaTime)
{
super.PlayerTick(DeltaTime);
if (bPlay)
  RunTime+=DeltaTime;
}

//needed for Levelname in Main Menu - and just a good thing to have
function Trigger(Actor Other, Pawn EventInstigator)
{
SaveConfig();
}

// ********************************************************************************
// Mouse movement, initialisation and execution of commmand line options
// ********************************************************************************

state MSBCode
{
  function PlayerTick(float DeltaTime)
  {
  global.playerTick(DeltaTime);
  //2D Positioning
  if (Mouse!=None && !Mouse.bTyping)
   {
    Mouse.xPos+=DeltaTime*MouseSens*aMouseX+aStrafe/6000;
    Mouse.yPos-=DeltaTime*MouseSens*aMouseY+aForward/6000;
    }
  }

Begin:
 if (MInteract==None)
 {
   Player.InteractionMaster.AddInteraction("MSBCode.MSBInterAction", Player);
   bHasInteraction=true;
   Mouse=MSBHud(myHud).MouseOverlay;
   MInteract.Mouse=MSBHud(myHud).MouseOverlay;
   Log("PlayerController spawns an Interaction");
   }
 if (bInstantLoad)
   {
   Log("instantLevel"@instantLevel);
   sleep(1);
   Load(instantLevel);
   sleep(1);
   bInstantLoad=false;
   }
 if (bInstantRun)
   {
   RunMovie();
   bInstantRun=false;
   }
}

// ********************************************************************************
// MoveObject used for posessing Cameras and Pawns (tbd)
// ********************************************************************************

state MoveObject
{

  function PlayerMove(float DeltaTime)
  {
        local vector X,Y,Z;

        ViewTarget.GetAxes(Rotation,X,Y,Z);
        Acceleration = 0.02 * (aForward*X + aStrafe*Y + aUp*vect(0,0,1));
        UpdateRotation(DeltaTime, 1);
        ViewTarget.MoveSmooth(SpectateSpeed * Normal(Acceleration) * DeltaTime);
        ViewTarget.SetRotation(Rotation);
  }
}

// ********************************************************************************
// Starts the simulation
// calls 'initialize' and 'RunMovie' functions in MSB specific Actors
// hides all Actors that have bDisplayed=false
// ********************************************************************************

exec function RunMovie()
{
local MSBActor MSA;
local MSBPawn MSP;
local MSBInterpolationHelper I;
local MSBRootNode R;
local MSBScriptNode S;
local MSBScriptTriggerNode T;

bPlay=!bPlay;
Log("executing Start/Stop Script");

if (bPlay)          //start now!
  {
  Tag='running';
  RunTime=0;

 // Mouse.deSelectAll();

  foreach AllActors(class'MSBRootNode',R)
    if (R!=None)
      R.initialize();

  foreach AllActors(class'MSBScriptTriggerNode',T)
    if (T!=None)
      T.initialize();

  foreach AllActors(class'MSBScriptNode',S)
    if (S!=None)
      S.initialize();


  foreach AllActors(class'MSBActor',MSA)
    if (MSA!=None)
      {
      MSA.RunMovie();
      //MSA.Reset();
      if (!MSA.bDisplayed)
        MSA.bHidden=true;
      }

  foreach AllActors(class'MSBPawn',MSP)
    if (MSP!=None)
      {
      MSP.iniPosing();
      MSP.RunMovie();
      }

  MSBHud(myHUD).DrawOverlays=false;

  if (bStartRender)
    ConsoleCommand("recordmovie start filename="$filename@"Quality=1 Width=720 Height=540 fps=25");
  }
else                 // end now!
  {
  if (bStartRender)
    {
    ConsoleCommand("recordmovie stop");
    bStartRender=false;
    }
  Tag='constructing';

  foreach AllActors(class'MSBRootNode',R)
    if (R!=None)
      R.Reset();

  foreach AllActors(class'MSBScriptNode',S)
    if (S!=None)
      S.Reset();

  MSBHud(myHUD).DrawOverlays=true;

  foreach AllActors(class'MSBActor',MSA)
    if (MSA!=None)
      {
      MSA.bHidden=false;
      MSA.StopMovie();
      MSA.Reset();
      }
  foreach AllActors(class'MSBPawn',MSP)
    if (MSP!=None)
        MSP.clearPosing();

  foreach AllActors(class'MSBInterpolationHelper',I)
    {
    Log("Destroying:"@i);
    if (I!=None)
      I.Destroy();
    }
  }
}


//calculates the position on the grid
function calculateGrid(out vector gridPos)
{
gridPos.X=int(gridPos.X/gridSpacing)*gridSpacing;
gridPos.Y=int(gridPos.Y/gridSpacing)*gridSpacing;
gridPos.Z=int(gridPos.Z/gridSpacing)*gridSpacing;
}

// ********************************************************************************
// general Concole Commands - also used by script
// ********************************************************************************

//switches between flying and editing
exec function switchStates()
{
bMove=!bMove;
if (bMove)
  {
  disableMidLayer();
  GotoState('Spectating');
  }
else
  {
  enableMidLayer();
  GotoState('MSBCode');
  }
}

//enables/disables the light grey layer in editing/scripting mode
exec function disableMidLayer()
{
MSBSceneEditorHud(myHud).DrawMidLayer=false;
}
exec function enableMidLayer()
{
MSBSceneEditorHud(myHud).DrawMidLayer=true;
}

exec function ToggleGrid()
{
bGrid=!bGrid;
}

exec function SetGridSpacing(int spacing)
{
GridSpacing=spacing;
}

exec function SaveGeneralConfig()
{
SaveConfig();
Log("General Config written!");
}

//destroy everything
exec function CleanUp()
{
local int i;
local ScriptedSequence S;

//remove Interaction
MInteract.M=None;
MInteract.Mouse=None;
Player.InteractionMaster.RemoveInteraction(MInteract);
Log("removing Interaction"@MInteract);
MInteract=None;
bHasInteraction=false;
MSBHud(myHud).DoFOverlay.Destroy();
//destroy all actions
foreach AllActors(class'ScriptedSequence', S)
  for (i=0;i<S.Actions.length;i++)
    S.Actions[i]=None;

// clear LevelNode array
for (i=0;i<LevelNodes.length;i++)
  LevelNodes[i]=None;
}

//Load a map - used with bInstantRun
exec function Load(string loadName)
{
local MSBSceneLoadButton SL;
local int i;

SL=Spawn(class'MSBSceneLoadButton',self);
SL.LoadMSBSet(loadname,true);
for (i=0;i<myHud.overlays.length;i++)
  if (myHud.overlays[i]==SL)
    myHud.Overlays.Remove(i,1);
SL.destroy();
}

//Used for RenderToDisk
exec function startRender (string fn)
{
bStartRender=true;
filename=fn$".avi";
RunMovie();
}

//creates an Actor at the Mouose Position
exec function placeMSBActor(string SpawnActorClassName)
{
local MSBActor SpawnActor;
local vector x,y,z;
local class <MSBActor> SpawnActorClass;

if (SpawnActorClass==None)
  SpawnActorClass = class<MSBActor>(DynamicLoadObject(SpawnActorClassName, class'Class'));
SpawnActor = Spawn(SpawnActorClass,,,Location,Rotation);

LevelActors.Insert(LevelActors.Length,1);
LevelActors[LevelActors.Length-1]=SpawnActor;
GetAxes(Rotation,x,y,z);
SetLocation(SpawnActor.Location-X*100);
}

//used for ContextMenu -right MouseButton
exec function MousePlaceMSBActor(string SpawnActorClassName)
{
local MSBActor SpawnActor;
local class <MSBActor> SpawnActorClass;

if (SpawnActorClass==None)
  SpawnActorClass = class<MSBActor>(DynamicLoadObject(SpawnActorClassName, class'Class'));
SpawnActor = Spawn(SpawnActorClass,,,Mouse.MouseLast3D,Rotation);

LevelActors.Insert(LevelActors.Length,1);
LevelActors[LevelActors.Length-1]=SpawnActor;
}

//creates a Keyframe a Mouse Position
exec function placeKeyFrame()
{
local MSBActor KeyActor;
local vector x,y,z;
local int i;

for (i=0;i<Mouse.selectedActor.length;i++)
  {
  KeyActor=MSBActor(Mouse.selectedActor[i]);
  KeyActor.Keys[KeyActor.Keys.length]=Spawn(class'MSBKeyFrame',KeyActor,,Location);
  KeyActor.Keys[KeyActor.Keys.length-1].KeyNumber=KeyActor.Keys.length-1;
  LevelActors[LevelActors.Length]=KeyActor.Keys[KeyActor.Keys.length-1];
  }

GetAxes(Rotation,x,y,z);
SetLocation(Location-X*100);
}

//contains Nodes into a container
exec function Contain()
{
local int i, currentNode;
local MSBContainerButton C;

C= Spawn(class'MSBContainerButton',self);
C.xPos=20;
C.yPos=500;

for (currentNode=0;currentNode<LevelNodes.length;currentNode++)
  if (LevelNodes[currentNode].bIsSelected)
    C.containedNodes[i++]=currentNode;

for (currentNode=0;currentNode<LevelMSBScriptNodes.length;currentNode++)
  if (LevelMSBScriptNodes[currentNode].bIsSelected)
    C.containedScriptNodes[i++]=currentNode;

for (currentNode=0;currentNode<LevelContainers.length;currentNode++)
  if (LevelContainers[currentNode].bIsSelected)
    C.containedContainers[i++]=currentNode;

C.hideContent(true);
Mouse.deSelectAll();
}


//**************************************************************************
//* MiniMap stuff                                                          *
//**************************************************************************
exec function showMiniMap()
{
if (MiniMap!=None)
  MiniMap.Destroy();
else
  MiniMap=Spawn(class'MSBMiniMap',self);
}

exec function shiftLeft(float left)
{
local int i;

scrollLeft+=left;

for (i=0;i<LevelNodes.length;i++)
  if (LevelNodes[i]!=None)
    LevelNodes[i].xPos-=left;

for (i=0;i<LevelMSBScriptNodes.length;i++)
  if (LevelMSBScriptNodes[i]!=None)
    LevelMSBScriptNodes[i].xPos-=left;

for (i=0;i<Levelcontainers.length;i++)
  if (Levelcontainers[i]!=None)
    Levelcontainers[i].xPos-=left;
}

exec function shiftUp(float up)
{
local int i;

scrollUp+=up;

for (i=0;i<LevelNodes.length;i++)
  if (LevelNodes[i]!=None)
    LevelNodes[i].yPos-=up;

for (i=0;i<LevelMSBScriptNodes.length;i++)
  if (LevelMSBScriptNodes[i]!=None)
    LevelMSBScriptNodes[i].yPos-=up;

for (i=0;i<Levelcontainers.length;i++)
  if (Levelcontainers[i]!=None)
    Levelcontainers[i].yPos-=up;
}


//**************************************************************************
//* Convert old Save/Load Structure to the new config based version        *
//**************************************************************************
exec function convertAll()
{
convert2DChar();
convertPoses();
convertSaveGames();
convertVoxelChars();
convertVoxelParts();
SaveGeneralConfig();
}

exec function convertPoses()
{
local int s,i;
local string loadName;
local MSBAniKey A,B;
local MSBGeneralPrefs gc;

gc=Level.Game.LoadDataObject(class'MSBGeneralPrefs',"GeneralConfig","GeneralConfig");
for (s=0;s<gc.SavePoses.length;s++)
  {
  loadName=gc.SavePoses[s];
  log (loadName);

  for (i=0;i<26;i++)
    {
    A=Level.Game.LoadDataObject(class'MSBAniKey',loadName$"Key"$i,"MovieSandboxPoses");
    B=Level.Game.CreateDataObject(class'MSBAniKey',loadName$"Key"$i,"MSBPose_"$loadName);
    B.BoneRotation=A.BoneRotation;
    B.OriginOffset=A.OriginOffset;
    }
  Level.Game.SavePackage("MSBPose_"$loadName);
  }
}

exec function convertSaveGames()
{
local MSBDataObject Load, Save;
local int s,i;
local MSBGeneralPrefs gc;

gc=Level.Game.LoadDataObject(class'MSBGeneralPrefs',"GeneralConfig","GeneralConfig");

for (s=0;s<gc.SaveGames.length;s++)
    {
    Log(gc.SaveGames[s]);
    Load=Level.Game.LoadDataObject(class'MSBDataObject',gc.SaveGames[s],"MSBSavedScenes");
    Save=Level.Game.CreateDataObject(class'MSBDataObject',gc.SaveGames[s],"MSBSave_"$gc.SaveGames[s]);

    for (i=0;i<Load.myComplexInfos.length;i++)
      Save.myComplexInfos[i]=Load.myComplexInfos[i];

    for (i=0;i<Load.myNodeInfos.length;i++)
      Save.myNodeInfos[i]=Load.myNodeInfos[i];

    for (i=0;i<Load.myMSBScriptNodeInfos.length;i++)
      Save.myMSBScriptNodeInfos[i]=Load.myMSBScriptNodeInfos[i];

    for (i=0;i<Load.myContainerInfos.length;i++)
      Save.myContainerInfos[i]=Load.myContainerInfos[i];

    for (i=0;i<Load.myActorInfos.length;i++)
      Save.myActorInfos[i]=Load.myActorInfos[i];

    for (i=0;i<Load.myPawnInfos.length;i++)
      Save.myPawnInfos[i]=Load.myPawnInfos[i];

    for (i=0;i<Load.mySpecialInfos.length;i++)
      Save.mySpecialInfos[i]=Load.mySpecialInfos[i];

    Save.pcLocation=Load.pcLocation;
    Save.pcRotation=Load.pcRotation;

    Level.Game.SavePackage("MSBSave_"$gc.SaveGames[s]);
    }
}

exec function convertVoxelChars()
{
local MSBVoxelCharacterSave Load,Save;
local int s;
local MSBGeneralPrefs gc;

gc=Level.Game.LoadDataObject(class'MSBGeneralPrefs',"GeneralConfig","GeneralConfig");

for (s=0;s<gc.SaveCharacters.length;s++)
  {
  Load=Level.Game.LoadDataObject(class'MSBVoxelCharacterSave',gc.SaveCharacters[s],"Char"$gc.SaveCharacters[s]);
  Save=Level.Game.CreateDataObject(class'MSBVoxelCharacterSave',gc.SaveCharacters[s],"MSBVoxChar_"$gc.SaveCharacters[s]);
  Save.Parts=Load.Parts;
  Level.Game.SavePackage("MSBVoxChar_"$gc.SaveCharacters[s]);
  }
}

exec function convert2DChar()
{
local MSB2DCharacterSave Load,Save;
local int s;
local MSBGeneralPrefs gc;

gc=Level.Game.LoadDataObject(class'MSBGeneralPrefs',"GeneralConfig","GeneralConfig");

for (s=0;s<gc.Save2DCharacters.length;s++)
    {
    Load=Level.Game.LoadDataObject(class'MSB2DCharacterSave',gc.Save2DCharacters[s],"MSB2DChar"$gc.Save2DCharacters[s]);
    Save=Level.Game.CreateDataObject(class'MSB2DCharacterSave',gc.Save2DCharacters[s],"MSB2DChar_"$gc.Save2DCharacters[s]);

    Save.Idle=Load.Idle;
    Save.Move=Load.Move;
    Save.Animations=Load.Animations;

    Save.headLocation=Load.headLocation;
    Save.headSize=Load.headSize;

    Save.MatHead=Load.MatHead;
    Save.MatMask=Load.MatMask;
    Save.MatEyes=Load.MatEyes;
    Save.MatEyeBrows=Load.MatEyeBrows;
    Save.MatMouthOpen=Load.MatMouthOpen;
    Save.MatMaskMouthOpen=Load.MatMaskMouthOpen;

    Save.CollisonHeight=Load.CollisonHeight;
    Save.CollisionRadius=Load.CollisionRadius;

    Level.Game.SavePackage("MSB2DChar_"$gc.Save2DCharacters[s]);
    }
}

exec function convertVoxelParts()
{
local MSBBlockCloud Load,Save;
local int s,i;
local MSBGeneralPrefs gc;

gc=Level.Game.LoadDataObject(class'MSBGeneralPrefs',"GeneralConfig","GeneralConfig");

for (s=0;s<gc.SaveCharParts.length;s++)
    {
    Load=Level.Game.LoadDataObject(class'MSBBlockCloud',gc.SaveCharParts[s],"CharacterParts");
    Save=Level.Game.CreateDataObject(class'MSBBlockCloud',gc.SaveCharParts[s],"MSBVoxPart_"$gc.SaveCharParts[s]);

    for (i=0;i<Load.BlockRelLocation.length;i++)
      Save.BlockRelLocation[i]=Load.BlockRelLocation[i];

    for (i=0;i<Load.Col.length;i++)
      Save.Col[i]=Load.Col[i];

    for (i=0;i<Load.BoneNumber.length;i++)
      Save.BoneNumber[i]=Load.BoneNumber[i];

    for (i=0;i<Load.drawScale.length;i++)
      Save.drawScale[i]=Load.drawScale[i];

    for (i=0;i<Load.drawScale3D.length;i++)
      Save.drawScale3D[i]=Load.drawScale3D[i];

    for (i=0;i<Load.BlockShader.length;i++)
      Save.BlockShader[i]=Load.BlockShader[i];

    for (i=0;i<Load.BoneRelLocation.length;i++)
      Save.BoneRelLocation[i]=Load.BoneRelLocation[i];

    for (i=0;i<Load.BoneRelRotation.length;i++)
      Save.BoneRelRotation[i]=Load.BoneRelRotation[i];
    Level.Game.SavePackage("MSBVoxPart_"$gc.SaveCharParts[s]);
    }
}

// ********************************************************************************
// debug and "soon to be implemented" things
// ********************************************************************************

exec function warningmessage(string warning)
{
local MSBWarningMessage wooh;

if (!bWarningMessageDisplayed)
  {
  wooh=Spawn(class'MSBwarningMessage',self);
  wooh.BtnName=warning;
  bWarningMessageDisplayed=true;
  }
}

exec function SetBrushSize(float size)
{
local MSBSimpleBlock block;

foreach allactors (class'MSBsimpleBlock',block)
  block.SetDrawScale(size);
}

//Crouching here!!!
exec function setKneeling(optional MSBPawn P)
{
local int oldRadius, oldHeight, i;

if (P==None)
 {
 for (i=0;i<Mouse.selectedActor.length;i++)
  {
  oldRadius=Mouse.selectedActor[i].CollisionRadius;
  oldHeight=Mouse.selectedActor[i].CollisionHeight;
  Mouse.selectedActor[i].SetCollisionSize(oldRadius/2,oldHeight/2);
  }
 }
else
 {
  oldRadius=P.CollisionRadius;
  oldHeight=P.CollisionHeight;
  P.SetCollisionSize(oldRadius/2,oldHeight/2);
 }
}

exec function setStanding()
{
local int oldRadius, oldHeight, i;
for (i=0;i<Mouse.selectedActor.length;i++)
  {
  oldRadius=Mouse.selectedActor[i].CollisionRadius;
  oldHeight=Mouse.selectedActor[i].CollisionHeight;
  Mouse.selectedActor[i].SetCollisionSize(oldRadius*2,oldHeight*2);
  }
}

//set Blocks to standard Voxelshader
//exec function BlocksToBrushes()
//{
//local MSBSimpleBlock block;
//
//foreach allactors (class'MSBsimpleBlock',block)
//  {
//  if (block.DrawType!=DT_None)
//   {
//   if (block.Shader==None)
//     {
//     block.Shader=new class'MSBBrushShader';
//     block.shader.Parent=block;
//     }
//   else
//     {
//     block.shader.Reset();
//     block.Shader=None;
//     }
//   }
//  }
//}

exec function SetDebug()
{
local MSBActor A;

foreach allactors (class'MSBActor',A)
 if (A!=None)
   A.SetDebug();
}



defaultproperties
{
DesiredFOV=90
DefaultFOV=90
InitialState=MSBCode
bHasInteraction=true
MouseSens=0.7
bGrid=true
Gridspacing=4
SpectateSpeed=100
canSelect=true


ContextButtonList(0)="MSBPropertyButtons.MSBPropertyButtonCommand"
ContextButtonList(1)="MSBPropertyButtons.MSBPropertyButtonCommand"
ContextButtonList(2)="MSBPropertyButtons.MSBPropertyButtonCommand"

ContextButtonCommand(0)="MousePlaceMSBActor MSBObjects.MSBMoveTargetActor"
ContextButtonCommand(1)="MousePlaceMSBActor MSBObjects.MSBTargetActor"
ContextButtonCommand(2)="SaveThis"

ContextButtonName(0)="Move Target"
ContextButtonName(1)="General Target"
ContextButtonName(2)="Save"

//hack: important for Ragdoll Marionette!
Skins(0)=Texture'DemoPlayerSkins.Jakob_Body'
Skins(1)=Texture'DemoPlayerSkins.Jakob_NewHead'
}

