class MSBPawn extends xPawn
config (BlockPawn);

#exec OBJ LOAD FILE=..\Animations\MSB_Reference.ukx

//***************************************************************************
// Posing
var() MSBAniKey PoseKey[63];
var int maxBones;
var float BlendTime;

var int LastPoseBtn;
var string PoseName[16];
var Actor MSBPawnInputControl;

//***************************************************************************
// Animation and Properties
var array<string> Aniname;
var array<string> ButtonList;
var array<string> Properties;

//***************************************************************************
// Mesh - acctual appearance
var Actor AttachedTo;
var Mesh UTMesh;
var string content;
var array<xUtil.PlayerRecord> PlayerList;

//***************************************************************************
// SkinSwapping
var string Head, Eyes, Eyebrows, Mask, MouthOpen, MaskMouthOpen;

var Material MatHead;
var Material MatMask;
var Material MatEyes;
var Material MatEyeBrows;
var Material MatMouthOpen;
var Material MatMaskMouthOpen;
var TexScaler moveEyes;
var TexScaler moveBrows;
var Combiner EyesAndHead;
var Combiner HeadAndBrows;
var Shader finishedHead;

var() float eyeX, eyeY,
            browX, browY;

var MSBUdpLink LipSyncUDP;
var int channel;
var int Oldchannel;
var int UdpPadCh;
var int OldUdpPadCh;

var bool bTalkable;
var bool bSkinSwap;

// *******************************************************************************
// general Setup
// *******************************************************************************

event PostBeginPlay()
{
super.PostBeginPlay();
}

simulated function AssignInitialPose()
{
super.AssignInitialPose();
TweenAnim(IdleRestAnim,0.0);
LinkMesh(UTMesh,true); //assign Reference Animations to the Pawn Mesh
}

function LoadCharacter()
{
local Mesh contentMesh;
local int i;

class'xUtil'.static.GetPlayerList(PlayerList);
for (i=0;i<PlayerList.length;i++)
  if (content==PlayerList[i].MeshName)
    {
    Skins[0]=Material(DynamicLoadObject(PlayerList[i].BodySkinName, class'Material'));
    Skins[1]=Material(DynamicLoadObject(PlayerList[i].FaceSkinName, class'Material'));
    Head=PlayerList[i].FaceSkinName;
    MouthOpen=PlayerList[i].FaceSkinName$"_MouthOpen";
    }
contentMesh= Mesh(DynamicLoadObject(content, class'Mesh'));
LinkMesh(contentMesh,true); //assign Reference Animations to the Pawn Mesh
PlayerList.Remove(0,PlayerList.length);
}

function RunMovie();

function Tick(float DeltaTime)
{
super.Tick(deltaTime);

if (bTalkable)
  Mimic();
}

// *******************************************************************************
// Lipsyncing and Gamepad Control
// *******************************************************************************


function Mimic()
{
if (bSkinSwap)
  {
  if (moveEyes!=None)
    {
    moveEyes.UOffset=eyeX;
    moveEyes.VOffset=eyeY;
    }
  if (moveBrows!=None)
    {
    moveBrows.UOffset=browX;
    moveBrows.VOffset=browY;
    }
  }
}

event Trigger(Actor Other, Pawn EventInstigator)
{
// When Triggered by LipSyncUdp, talk
if (Other==LipSyncUDP && bTalkable)
  {
  Talk();
  SetTimer(0.2,false);
  }

// when assigning a channel, we initialise Lipsyncing
if (channel!=OldChannel)
  {
  iniLipSync();
  bTalkable=true;
  Log("Changed Channel in Pawn to"@channel);
  OldChannel=Channel;
  }

// UDP Gamepad control and channel
if (MSBPawnInputControl!=None && UdpPadCh!=OldUdpPadCh)
  {
  MSBPawnInputControl.Trigger(Other,self);
  Log("Changed UdpPadChannel in Pawn to"@UdpPadCh);
  OldUdpPadCh=UdpPadCh;
  }
}

// *************************************
// change the Mouth
// completely overwrite this for other
// types of Lipsyncing

function talk()
{
if (bSkinSwap && MatMouthOpen!=None && EyesAndHead.Material2!=MatMouthOpen)
  {
  EyesAndHead.Material2=MatMouthOpen;
  EyesAndHead.Mask=MatMouthOpen;
 // finishedHead.Opacity=MatMaskMouthOpen;  --generates a sorting bug and is not used.
  }
}


// *************************************
// initialize Lipsyncing
// append other Lipsync types in this
// function.
function iniLipSync()
{
initUdpLink();
if (bSkinSwap)
  iniSkinSwap();
}

function Timer()
{
if (bSkinSwap)
  {
  EyesAndHead.Material2=MatHead;
  EyesAndHead.Mask=MatHead;
//  finishedHead.Opacity=MatMask;            --generates a sorting bug and is not used.
  Texture(MatMouthOpen).AnimCurrent=texture(MatMouthOpen);
  texture(MatMouthOpen).PrimeCount=0;
  texture(MatMouthOpen).PrimeCurrent=0;
  }
}

function initUdpLink()
{
Log("init Pawn Skinswapping");
if (LipSyncUDP!=None)
  LipSyncUDP.Destroy();

//todo: changed to accomodate for unified UDP data Object
LipSyncUDP=Spawn(class'MSBUdpLink',self);
LipSyncUDP.InitUdpLink(21840+channel);
}

function iniSkinSwap()
{
MatHead=Material(DynamicLoadObject(Head,class'Material',true));
MatMask=Material(DynamicLoadObject(Mask,class'Material',true));
MatMouthOpen=Material(DynamicLoadObject(MouthOpen,class'Material',true));

if (MatMouthOpen==None)
  MatMouthOpen=Material(DynamicLoadObject(default.MouthOpen,class'Material',true));

MatMaskMouthOpen=Material(DynamicLoadObject(MaskMouthOpen,class'Material',true));
MatEyes=Material(DynamicLoadObject(Eyes,class'Material',true));
MatEyeBrows=Material(DynamicLoadObject(Eyebrows,class'Material',true));

if (moveEyes==None)
  moveEyes=new class'TexScaler';
moveEyes.Material=MatEyes;

if (moveBrows==None)
  moveBrows=new class'TexScaler';
moveBrows.Material=MatEyeBrows;

if (EyesAndHead==None)
  EyesAndHead=new class'Combiner';
EyesAndHead.Material1=moveEyes;
EyesAndHead.Material2=MatHead;
EyesAndHead.Mask=MatHead;
EyesAndHead.CombineOperation=CO_AlphaBlend_With_Mask;

if (HeadAndBrows==None)
  HeadAndBrows=new class'Combiner';
HeadAndBrows.Material1=moveBrows;
HeadAndBrows.CombineOperation=CO_AlphaBlend_With_Mask;
HeadAndBrows.InvertMask=true;
HeadAndBrows.Mask=moveBrows;
HeadAndBrows.Material2=EyesAndHead;
if (Eyebrows=="")
  {
  HeadAndBrows.Material1=MatMask;
  HeadAndBrows.CombineOperation=CO_Use_Color_From_Material2;
  HeadAndBrows.Mask=MatMask;
  Log("No EyeBrows present, switching eyebrows off!");
  }


if (finishedHead==None)
  finishedHead=new class'Shader';
finishedHead.Diffuse=HeadAndBrows;
// no mask --generates a sorting bug and is not used.

Skins[1]=finishedHead;
RepSkin=finishedHead;
}

// *******************************************************************************
// Posing and Animation
// *******************************************************************************

function iniPosing()
{
local int i;

for (i=0;i<maxBones;i++)
  {
  if (PoseKey[i]==None)
    {
    PoseKey[i]=new class'MSBAniKey';
    PoseKey[i].BoneRotation=Rotation*0;
    PoseKey[i].OriginOffset=Location*0;
    }
  PoseKey[i].BoneNumber=i;
  PoseKey[i].KeyActor=self;
  }
Log("Posing initialised");
}

function clearPosing()
{
local int i;

if (PoseKey[6]!=None)
  {
  for (i=0;i<maxBones;i++)
    {
    PoseKey[i].KeyActor=self;
    PoseKey[i].BoneRotation=Rotation*0;
    PoseKey[i].changeBoneRotation();
    PoseKey[i].OriginOffset=Location*0;
    PoseKey[i].changeBoneLocation();
    PoseKey[i].BoneNumber=0;
    PoseKey[i]=None;
    }
  }
}

function setPawnPose(string poseName)
{
local int i;
local MSBInterpolationHelper IH;
local MSBAniKey A;
local MSBAniKey TweenKey[63];
local int myMaxBones;

//leave the head unaffected by Pose changes if we control this Character with a gamepad!
if (UdpPadCh>0)
  myMaxBones=maxBones-1;

for (i=0;i<myMaxBones;i++)
 {
 A=Level.Game.LoadDataObject(class'MSBAniKey',PoseName$"Key"$i,"MSBPose_"$PoseName);
 TweenKey[i]=new class'MSBAniKey';
 TweenKey[i].BoneNumber=i;
 if (A!=None)
   TweenKey[i].BoneRotation=A.BoneRotation;
 }

if (BlendTime==0)
 {
 for (i=0;i<myMaxBones;i++)
  {
  TweenKey[i].KeyActor=self;
  TweenKey[i].changeBoneRotation();
  }
 }
 else
 {
 for (i=0;i<myMaxBones;i++)
  {
  TweenKey[i].KeyActor=self;
  IH=Spawn(class'MSBInterpolationHelper');
  IH.TargetKey=TweenKey[i];
  IH.blendTime=BlendTime;
  IH.initializePoseTween();
  }
 }
}

function clearPawn()
{
MSBPawnInputControl.Destroy();
}

defaultproperties
{
BlendTime=0.2
bSkinSwap=true

Head="DemoPlayerSkins.Jakob_NewHead"
Mask="MovieSandboxTextures.White"
MouthOpen="MovieSandboxTextures.HeadTex.Jakob_NewHead_MouthOpn_a02"
MaskMouthOpen="MovieSandboxTextures.White"
Eyes="MovieSandboxTextures.White"
EyeBrows=""

bDeferRendering=false

Mesh=Mesh'MSB_Reference.male'                                 //base animation resource
UTMesh=Mesh'ThunderCrash.JakobM'

ButtonList(0)="MSBPropertyButtons.MSBPropertyButtonMove"
Properties(0)="Dummy"
ButtonList(1)="MSBPropertyButtons.MSBPropertyButtonRotate"
Properties(1)="Dummy"
ButtonList(2)="MSBPropertyButtons.MSBPropertyButtonScale"
Properties(2)="Dummy"
ButtonList(3)="MSBPropertyButtons.MSBPropertyButtonAdvancedProps"
Properties(3)="Dummy"
ButtonList(4)="MSBPropertyButtons.MSBPropertyButtonText"
Properties(4)="Channel"
ButtonList(5)="MSBPropertyButtons.MSBPropertyButtonSpawnPadButton"
Properties(5)="Pad"
ButtonList(6)="MSBPropertyButtons.MSBPropertyButtonText"
Properties(6)="UdpPadCh"

Aniname(0)="pthrust"
Aniname(1)="asssmack"
Aniname(2)="throatcut"
Aniname(3)="gesture_beckon"
Aniname(4)="gesture_cheer"
Aniname(5)="gesture_halt"
Aniname(6)="gesture_point"
Aniname(7)="specific_1"

maxBones=26

Physics=PHYS_Falling

SoundGroupClass=None
GibGroupClass=None

BaseEyeHeight=70.0
EyeHeight=70.0
CollisionRadius=20
CollisionHeight=48.0
GroundSpeed=180.0
AirSpeed=300.0
WaterSpeed=220.0
DodgeSpeedFactor=1.5
DodgeSpeedZ=210.0
AccelRate=2048.0
JumpZ=500.0
CrouchHeight=29.0
CrouchRadius=25.0
bPlayOwnFootsteps=true
SoundRadius=0
SoundVolume=0



LODBias=1.8
MovementAnims(0)=RunF
MovementAnims(1)=RunB
MovementAnims(2)=TurnL
MovementAnims(3)=TurnR
SwimAnims(0)=SwimF
SwimAnims(1)=SwimB
SwimAnims(2)=SwimL
SwimAnims(3)=SwimR
CrouchAnims(0)=CrouchF
CrouchAnims(1)=CrouchB
CrouchAnims(2)=CrouchL
CrouchAnims(3)=CrouchR
WalkAnims(0)=WalkF
WalkAnims(1)=WalkB
WalkAnims(2)=TurnL
WalkAnims(3)=TurnR
AirStillAnim=Idle
AirAnims(0)=Idle
AirAnims(1)=Idle
AirAnims(2)=Idle
AirAnims(3)=Idle
TakeoffStillAnim=Idle
TakeoffAnims(0)=Idle
TakeoffAnims(1)=Idle
TakeoffAnims(2)=Idle
TakeoffAnims(3)=Idle
LandAnims(0)=Idle
LandAnims(1)=Idle
LandAnims(2)=Idle
LandAnims(3)=Idle
DoubleJumpAnims(0)=Idle
DoubleJumpAnims(1)=Idle
DoubleJumpAnims(2)=Idle
DoubleJumpAnims(3)=Idle
DodgeAnims(0)=Idle
DodgeAnims(1)=Idle
DodgeAnims(2)=Idle
DodgeAnims(3)=Idle
WallDodgeAnims(0)=Idle
WallDodgeAnims(1)=Idle
WallDodgeAnims(2)=Idle
WallDodgeAnims(3)=Idle

TurnRightAnim=TurnR
TurnLeftAnim=TurnL
CrouchTurnRightAnim=TurnR
CrouchTurnLeftAnim=TurnR
IdleRestAnim=Idle
IdleCrouchAnim=Crouch
IdleSwimAnim=Swim_Tread
IdleWeaponAnim=Idle
IdleHeavyAnim=Idle
IdleRifleAnim=Idle
FireHeavyRapidAnim=Biggun_Burst
FireHeavyBurstAnim=Biggun_Aimed
FireRifleRapidAnim=Rifle_Burst
FireRifleBurstAnim=Rifle_Aimed


RequiredEquipment(0)="None"
RequiredEquipment(1)="None"

ScaleGlow=1.0
AmbientGlow=40

TeleportFXClass=None

GibCountCalf=0
GibCountForearm=0
GibCountHead=0
GibCountTorso=0
GibCountUpperArm=0

bDoTorsoTwist=true

bActorShadows=true
bPlayerShadows=true

ShieldHitMat=None
UDamageWeaponMaterial=none
InvisMaterial=None


MaxMultiJump=1
MultiJumpRemaining=0
MultiJumpBoost=0

RagdollLifeSpan=0

RagImpactSounds(0)=none
RagImpactSounds(1)=none
RagImpactSounds(2)=none

VoiceType="None"
UDamageSound=None
FootstepVolume=0.15
TransOutEffect(0)=None
TransOutEffect(1)=None
TransEffects(0)=None
TransEffects(1)=None
PlacedCharacterName="MSBPawn"
PlacedFemaleCharacterName="none"

MaxLights=8
bDramaticLighting=true

bCanDodgeDoubleJump=false
bCanWallDodge=false
bNoCoronas=false
}
