class MSBVoxelEdController extends MSBController;

//voxel related
var MSBBlockBrush DrawBrush;
var MSBVoxelEdPawn myReferencePawn;


//internal - used for Moving BlockBrush around when in "Run" Mode
var vector xAxis,yAxis,zAxis;
var int oldX,oldY;

var vector nullVect;
var rotator nullrot;

//switch between VoxelChar and VoxelProp
var bool bCharEd;

//debug selfShader
var() float u,v,sf;

event PostBeginPlay()
{
super.PostBeginPlay();

//get rid of context buttons
ContextButtonList.remove(0,ContextButtonList.Length);

//set up DrawBrush
DrawBrushSetup();

//set up Character and stuff
switchMode();

//set view posititon
SetLocation(StartLoc);
SetRotation(StartRot);

//set Master Blocks to look skeleton-like
refreshMasters();
}

function DrawBrushSetup()
{
local float flGridSpc;
DrawBrush=Spawn(class'MSBBlockBrush',self,,nullVect);
flGridSpc=GridSpacing;
drawBrush.SetDrawScale(flGridSpc/8);
}

state CharacterEditor
{
  function PlayerTick(float DeltaTime)
  {
  global.playerTick(DeltaTime);
  //2D Positioning
  if (Mouse!=None && !Mouse.bTyping)
   {
    Mouse.xPos+=DeltaTime*MouseSens*aMouseX+aStrafe/6000;
    Mouse.yPos-=DeltaTime*MouseSens*aMouseY+aForward/6000;
    }
  }

Begin:
 if (MInteract==None)
   {
   Player.InteractionMaster.AddInteraction("MSBCode.MSBInteraction", Player);
   bHasInteraction=true;
   Mouse=MSBHud(myHud).MouseOverlay;
   MInteract.Mouse=MSBHud(myHud).MouseOverlay;
   Log("PlayerController spawns an Interaction");
   }
}

//switches between Drawing and selecting mode
exec function RunMovie()
{
bPlay=!bPlay;
Log("switching Draw/select Mode");

refreshMasters();

if (bPlay)          //start now!
  {
  Mouse.bHitActors=false;
  oldX=Mouse.xPos;
  oldY=Mouse.yPos;
  GetAxes(Rotation,xAxis,yAxis,zAxis);
  xAxis.Z=0;
  yAxis.Z=0;
  MSBHud(myHud).DrawOverlays=false;
  }
else
  {
  Mouse.bHitActors=true;
  MSBHud(myHud).DrawOverlays=true;
  }
}

exec function switchStates()
{
bMove=!bMove;
if (bMove)
  GotoState('BaseSpectating');
else
  GotoState('CharacterEditor');
}

//switches between creating a character and creating a Prop
exec function switchMode()
{
if (bCharEd)
  {
  if (myReferencePawn==None)
    {
    DrawBrush.Master.clearBlocks();
    DrawBrush.Master.Destroy();
    myReferencePawn=Spawn(class'MSBVoxelEdPawn',,,nullvect);
    myReferencePawn.SetPhysics(PHYS_None);
    myReferencePawn.SetRotation(Rotation*0);
    myReferencePawn.channel=1;
    myReferencePawn.Trigger(self,None);
    }
  DrawBrush.Master=myReferencePawn.Blocks[0];
  DrawBrush.SetRotation(0*Rotation);
  }
else
  {
  if (myReferencePawn!=None)
    {
    myReferencePawn.clearPawn();
    myReferencePawn.Destroy();
    }
  DrawBrush.Master=Spawn(class'MSBMasterBlock',,,nullvect);
  }
bCharEd=!bCharEd;
}
//don't spawn things for now...
exec function placeMSBActor(string SpawnActorClassName);

//resets the MasterBones to their drawscale to make them look like bones
exec function refreshMasters()
{
local int i;

for (i=0;i<myReferencePawn.Blocks.length;i++)
  if (myReferencePawn.Blocks[i]!=DrawBrush.Master)
    myReferencePawn.Blocks[i].SetDrawScale3D(myReferencePawn.Blocks[i].edScale);
}

//show/hide masterblocks
exec function ToggleMasters()
{
local MSBMasterBlock M;
foreach AllActors (class'MSBMasterBlock',M)
  M.bDisplayed=!M.bDisplayed;
Log(myReferencePawn);
}

exec function SetGridSpacing(int spacing)
{
local float flGridSpc;

super.SetGridSpacing(spacing);
flGridSpc=GridSpacing;
}

exec function calculateGridRelative(vector gridVec)
{
gridVec=gridVec*(GridSpacing*VSize(normal(gridVec)));
}

//testing
exec function SetBrushTo(string brushType)
{
DrawBrush.BlockClass=class<MSBSimpleBlock>(DynamicLoadObject(brushType,class'Class'));
}

exec function growBrush()
{
if (drawBrush.DrawScale<3)
  drawBrush.SetDrawScale(drawBrush.DrawScale+0.25);
}

exec function shrinkBrush()
{
if (drawBrush.DrawScale>0.3)
  drawBrush.SetDrawScale(drawBrush.DrawScale-0.25);
}

exec function ChangeMasterColor()
{
local int i;
for (i=0;i<DrawBrush.Master.S.length;i++)
  ColorModifier(DrawBrush.Master.S[i].Skins[0]).Color=ColorModifier(DrawBrush.Skins[0]).Color;
}

//get rid of all Blocks in MasterBone
exec function clearBlock()
{
MSBMasterBlock(MSBPropertyButton(Mouse.hudTargetActor).Parent).clearBlocks();
}

//set new FaceBlock to the last painted Block
exec function updateFaceBlocks()
{
myReferencePawn.MSBFaceBlock[myReferencePawn.MSBFaceBlock.length]=DrawBrush.Master.S[DrawBrush.Master.S.length-1];
Log("Updating Mouth in referencePawn...");
}

exec function moveToLimb(MSBMasterBlock MB);


defaultproperties
{
Gridspacing=2
bCharEd=true
StartLoc=(X=81,Y=-32,Z=-6)
StartRot=(Pitch=-524,Yaw=27500)
canSelect=false
}
